import { __extends, __values } from "tslib";
import * as React from 'react';
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { IgrToolActionCollection } from "./igr-tool-action-collection";
import { Visibility_$type } from "igniteui-react-core";
import { IgrToolCommandEventArgs } from "./igr-tool-command-event-args";
import { IgrToolActionPerformedEventArgs } from "./igr-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { TypeRegistrar } from "igniteui-react-core";
import { NamePatcher, getModifiedProps, isValidProp, brushToString, stringToBrush, ensureBool, ensureEnum, toSpinal, initializePropertiesFromCss } from "igniteui-react-core";
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { SyncableObservableCollection$1 } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
/**
 * Represents base class for all tool actions in toolbar
*/
var IgrToolAction = /** @class */ /*@__PURE__*/ (function (_super) {
    __extends(IgrToolAction, _super);
    function IgrToolAction(props) {
        var _this = _super.call(this, props) || this;
        _this._actions = null;
        _this.__p = null;
        _this._hasUserValues = new Set();
        _this._stylingContainer = null;
        _this._stylingParent = null;
        _this._inStyling = false;
        _this._onCommand = null;
        _this._onCommand_wrapped = null;
        _this._performed = null;
        _this._performed_wrapped = null;
        if (_this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(_this));
        }
        _this._implementation = _this.createImplementation();
        _this._implementation.externalObject = _this;
        _this.onImplementationCreated();
        if (_this._initializeAdapters) {
            _this._initializeAdapters();
        }
        return _this;
    }
    IgrToolAction.prototype.createImplementation = function () {
        return null;
    };
    Object.defineProperty(IgrToolAction.prototype, "i", {
        /**
         * @hidden
         */
        get: function () {
            return this._implementation;
        } /**
                                     * @hidden
                                     */,
        enumerable: false,
        configurable: true
    });
    IgrToolAction._createFromInternal = function (internal) {
        if (!internal) {
            return null;
        }
        if (!internal.$type) {
            return null;
        }
        var name = internal.$type.name;
        var externalName = "Igr" + name;
        if (!TypeRegistrar.isRegistered(externalName)) {
            return null;
        }
        return TypeRegistrar.create(externalName);
    };
    IgrToolAction.prototype.onImplementationCreated = function () {
    };
    IgrToolAction.prototype.componentDidMount = function () {
        var e_1, _a;
        try {
            for (var _b = __values(Object.keys(this.props)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var p = _c.value;
                if (isValidProp(this, p)) {
                    this[p] = this.props[p];
                }
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return))
                    _a.call(_b);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
    };
    IgrToolAction.prototype.shouldComponentUpdate = function (nextProps, nextState) {
        var e_2, _a;
        var mod = getModifiedProps(this.props, nextProps);
        try {
            for (var _b = __values(Object.keys(mod)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var p = _c.value;
                if (isValidProp(this, p)) {
                    this[p] = mod[p];
                }
            }
        }
        catch (e_2_1) {
            e_2 = { error: e_2_1 };
        }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return))
                    _a.call(_b);
            }
            finally {
                if (e_2)
                    throw e_2.error;
            }
        }
        return true;
    };
    IgrToolAction.prototype.render = function () {
        return null;
    };
    Object.defineProperty(IgrToolAction.prototype, "id", {
        get: function () {
            return this.i.cw;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actions", {
        /**
         * Gets the sub actions associated with this tool action.
        */
        get: function () {
            if (this._actions === null) {
                var coll = new IgrToolActionCollection();
                var innerColl = this.i.actions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._actions = coll._fromInner(innerColl);
            }
            return this._actions;
        },
        set: function (v) {
            if (this._actions !== null) {
                this._actions._setSyncTarget(null);
                this._actions = null;
            }
            var coll = new IgrToolActionCollection();
            this._actions = coll._fromOuter(v);
            var syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            var innerColl = this.i.actions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._actions._setSyncTarget(syncColl);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "width", {
        /**
         * Gets or sets the width to use for this tool action.
        */
        get: function () {
            return this.i.bl;
        },
        set: function (v) {
            this.i.bl = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "parentId", {
        get: function () {
            return this.i.c5;
        },
        set: function (v) {
            this.i.c5 = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "beforeId", {
        /**
         * Gets or sets the Id of the action that this action will be inserted before.
         * Subsequent actions will continue to insert before the target action, following
         * the original action that set this property. If using this property do not use the
         * AfterId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get: function () {
            return this.i.cc;
        },
        set: function (v) {
            this.i.cc = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "afterId", {
        /**
         * Gets or sets the Id of the action that this action will be inserted after.
         * Subsequent actions will continue to insert after the target action, following
         * the original action that set this property. If using this property do not use the
         * BeforeId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get: function () {
            return this.i.b9;
        },
        set: function (v) {
            this.i.b9 = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "overlayId", {
        /**
         * Sets to the Id of the action that this action will overlay itself on to.
         * The purpose of this property is to allow users to override actions automatically generated by
         * toolbar targets such as the DataChart. These generated actions all have unique Ids associated
         * with them that can be used for the OverlayId.
         * These overlaid actions will
         * not
         * replace the generated one but will have its properties
         * overlaid onto the generated actions.
        */
        get: function () {
            return this.i.c0;
        },
        set: function (v) {
            this.i.c0 = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "commandId", {
        get: function () {
            return this.i.ch;
        },
        set: function (v) {
            this.i.ch = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "commandArgument", {
        get: function () {
            return this.i.ce;
        },
        set: function (v) {
            this.i.ce = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "commandArgumentValue", {
        get: function () {
            return this.i.bn;
        },
        set: function (v) {
            this.i.bn = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "paddingLeft", {
        get: function () {
            return this.i.bh;
        },
        set: function (v) {
            this.i.bh = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualPaddingLeft", {
        get: function () {
            return this.i.a5;
        },
        set: function (v) {
            this.i.a5 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "paddingRight", {
        get: function () {
            return this.i.bi;
        },
        set: function (v) {
            this.i.bi = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualPaddingRight", {
        get: function () {
            return this.i.a6;
        },
        set: function (v) {
            this.i.a6 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "paddingBottom", {
        get: function () {
            return this.i.bg;
        },
        set: function (v) {
            this.i.bg = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualPaddingBottom", {
        get: function () {
            return this.i.a4;
        },
        set: function (v) {
            this.i.a4 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "paddingTop", {
        get: function () {
            return this.i.bj;
        },
        set: function (v) {
            this.i.bj = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualPaddingTop", {
        get: function () {
            return this.i.a7;
        },
        set: function (v) {
            this.i.a7 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualHoverBackground", {
        /**
         * Gets the actual hover background used by the tool action.
        */
        get: function () {
            return brushToString(this.i.eu);
        },
        set: function (v) {
            this.i.eu = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "hoverBackground", {
        /**
         * Gets or sets the hover background for this tool action.
        */
        get: function () {
            return brushToString(this.i.e0);
        },
        set: function (v) {
            this.i.e0 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualBackground", {
        /**
         * Gets the actual background used by the tool action.
        */
        get: function () {
            return brushToString(this.i.es);
        },
        set: function (v) {
            this.i.es = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "background", {
        /**
         * Gets or sets the background color of the tool action.
        */
        get: function () {
            return brushToString(this.i.ey);
        },
        set: function (v) {
            this.i.ey = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualTextColor", {
        /**
         * Gets the actual label text color.
        */
        get: function () {
            return brushToString(this.i.ex);
        },
        set: function (v) {
            this.i.ex = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "textColor", {
        /**
         * Gets or sets the label text color.
        */
        get: function () {
            return brushToString(this.i.e3);
        },
        set: function (v) {
            this.i.e3 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualDisabledTextColor", {
        /**
         * Gets the actual label text color when the tool is disabled.
        */
        get: function () {
            return brushToString(this.i.et);
        },
        set: function (v) {
            this.i.et = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "disabledTextColor", {
        /**
         * Gets or sets the label text color when the tool is disabled.
        */
        get: function () {
            return brushToString(this.i.ez);
        },
        set: function (v) {
            this.i.ez = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualTextStyle", {
        /**
         * Gets the actual font used.
        */
        get: function () {
            if (this.i.r == null) {
                return null;
            }
            return this.i.r.fontString;
        },
        set: function (v) {
            var fi = new FontInfo();
            fi.fontString = v;
            this.i.r = fi;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "textStyle", {
        /**
         * Gets or sets the font to use.
        */
        get: function () {
            if (this.i.t == null) {
                return null;
            }
            return this.i.t.fontString;
        },
        set: function (v) {
            var fi = new FontInfo();
            fi.fontString = v;
            this.i.t = fi;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "isDisabled", {
        get: function () {
            return this.i.aj;
        },
        set: function (v) {
            this.i.aj = ensureBool(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "name", {
        get: function () {
            return this.i.cy;
        },
        set: function (v) {
            this.i.cy = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "title", {
        /**
         * Gets or sets the title of the tool.
        */
        get: function () {
            return this.i.de;
        },
        set: function (v) {
            this.i.de = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "subtitle", {
        /**
         * Gets or sets the subtitle of the tool.
        */
        get: function () {
            return this.i.da;
        },
        set: function (v) {
            this.i.da = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "iconName", {
        get: function () {
            return this.i.cp;
        },
        set: function (v) {
            this.i.cp = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "iconCollectionName", {
        get: function () {
            return this.i.cm;
        },
        set: function (v) {
            this.i.cm = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "iconViewBoxLeft", {
        /**
         * Gets or sets the icon viewbox left.
        */
        get: function () {
            return this.i.bd;
        },
        set: function (v) {
            this.i.bd = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "iconViewBoxTop", {
        /**
         * Gets or sets the icon viewbox top.
        */
        get: function () {
            return this.i.be;
        },
        set: function (v) {
            this.i.be = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "iconViewBoxWidth", {
        /**
         * Gets or sets the icon viewbox width.
        */
        get: function () {
            return this.i.bf;
        },
        set: function (v) {
            this.i.bf = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "iconViewBoxHeight", {
        /**
         * Gets or sets the icon viewbox height.
        */
        get: function () {
            return this.i.bc;
        },
        set: function (v) {
            this.i.bc = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "iconFill", {
        /**
         * Gets or sets the icon fill color.
        */
        get: function () {
            return brushToString(this.i.e1);
        },
        set: function (v) {
            this.i.e1 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualIconFill", {
        /**
         * Gets the actual icon fill color.
        */
        get: function () {
            return brushToString(this.i.ev);
        },
        set: function (v) {
            this.i.ev = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "iconStroke", {
        /**
         * Gets or sets the icon stroke color.
        */
        get: function () {
            return brushToString(this.i.e2);
        },
        set: function (v) {
            this.i.e2 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "actualIconStroke", {
        /**
         * Gets the actual icon stroke color.
        */
        get: function () {
            return brushToString(this.i.ew);
        },
        set: function (v) {
            this.i.ew = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "visibility", {
        /**
         * Gets or sets the tool action visibility.
        */
        get: function () {
            return this.i.e5;
        },
        set: function (v) {
            this.i.e5 = ensureEnum(Visibility_$type, v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "closeOnExecute", {
        /**
         * Gets or sets if the tool action will close its parent dropdown when executed by the user.
        */
        get: function () {
            return this.i.ae;
        },
        set: function (v) {
            this.i.ae = ensureBool(v);
        },
        enumerable: false,
        configurable: true
    });
    IgrToolAction.prototype.findByName = function (name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        return null;
    };
    Object.defineProperty(IgrToolAction.prototype, "hasUserValues", {
        get: function () {
            return this._hasUserValues;
        },
        enumerable: false,
        configurable: true
    });
    IgrToolAction.prototype.__m = function (propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    };
    IgrToolAction.prototype._styling = function (container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        var genericPrefix = "";
        var typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolAction");
        var additionalPrefixes = [];
        var prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        var b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            var basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            var parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            var parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    };
    IgrToolAction.prototype.cleanUp = function () {
        this.i.dh();
    };
    IgrToolAction.prototype.fromInfo = function (info) {
        this.i.dt(info);
    };
    IgrToolAction.prototype.openSubMenu = function () {
        this.i.d6();
    };
    IgrToolAction.prototype.closeSubmenu = function () {
        this.i.dj();
    };
    Object.defineProperty(IgrToolAction.prototype, "onCommand", {
        get: function () {
            return this._onCommand;
        },
        set: function (ev) {
            var _this = this;
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = function (o, e) {
                var outerArgs = new IgrToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnCommand) {
                    _this.beforeOnCommand(_this, outerArgs);
                }
                if (_this._onCommand) {
                    _this._onCommand(_this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolAction.prototype, "performed", {
        get: function () {
            return this._performed;
        },
        set: function (ev) {
            var _this = this;
            if (this._performed_wrapped !== null) {
                this.i.performed = delegateRemove(this.i.performed, this._performed_wrapped);
                this._performed_wrapped = null;
                this._performed = null;
            }
            this._performed = ev;
            this._performed_wrapped = function (o, e) {
                var outerArgs = new IgrToolActionPerformedEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforePerformed) {
                    _this.beforePerformed(_this, outerArgs);
                }
                if (_this._performed) {
                    _this._performed(_this, outerArgs);
                }
            };
            this.i.performed = delegateCombine(this.i.performed, this._performed_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    return IgrToolAction;
}(React.Component));
export { IgrToolAction };
